local dt = require "darktable"
local ds = require "lib/dtutils.string"
local log = require "lib/dtutils.log"

local img = dt.database.get_image(1)
local darktable_picture_folder = "/home/bill/Pictures/dttestlua_sub/Play.Raw"

local tests = {}
local expected = {}

local eyear,emon,eday,ehour,emin,esec,emsec
if dt.preferences.read("darktable", "lighttable/ui/milliseconds", "bool") then
  eyear,emon,eday,ehour,emin,esec,emsec = string.match(img.exif_datetime_taken, "(%d+):(%d+):(%d+) (%d+):(%d+):(%d+)%.(%d+)$")
else
  emsec = "0"
  eyear,emon,eday,ehour,emin,esec = string.match(img.exif_datetime_taken, "(%d+):(%d+):(%d+) (%d+):(%d+):(%d+)$")
end
local datetime = os.date("*t")

tests[1] = "$(ROLL.NAME)" 
tests[2] = "$(FILE.FOLDER)"
tests[3] = "$(FILE.NAME)"
tests[4] = "$(FILE.EXTENSION)" 
tests[5] = "$(ID)" 
tests[6] = "$(VERSION)" 
tests[7] = "$(VERSION.IF_MULTI)"    -- Not Implemented
tests[8] = "$(VERSION.NAME)"
tests[9] = "$(DARKTABLE.VERSION)"
tests[10] = "$(DARKTABLE.NAME)"      -- Not Implemented
tests[11] = "$(SEQUENCE)"
tests[12] = "$(WIDTH.SENSOR)"        -- Not Implemented
tests[13] = "$(HEIGHT.SENSOR)"       -- Not Implemented
tests[14] = "$(WIDTH.RAW)"           -- Not Implemented
tests[15] = "$(HEIGHT.RAW)"          -- Not Implemented
tests[16] = "$(WIDTH.CROP)"          -- Not Implemented
tests[17] = "$(HEIGHT.CROP)"         -- Not Implemented
tests[18] = "$(WIDTH.EXPORT)"        -- Not Implemented
tests[19] = "$(HEIGHT.EXPORT)"       -- Not Implemented
tests[20] = "$(WIDTH.MAX)"           -- Not Implemented
tests[21] = "$(HEIGHT.MAX)"         -- Not Implemented
tests[22] = "$(YEAR)"
tests[23] = "$(MONTH)"
tests[24] = "$(DAY)"
tests[25] = "$(HOUR)"
tests[26] = "$(MINUTE)"
tests[27] = "$(SECOND)"
tests[28] = "$(MSEC)"
tests[29] = "$(EXIF.YEAR)"
tests[30] = "$(EXIF.MONTH)"
tests[31] = "$(EXIF.DAY)"
tests[32] = "$(EXIF.HOUR)"
tests[33] = "$(EXIF.MINUTE)"
tests[34] = "$(EXIF.SECOND)"
tests[35] = "$(EXIF.MSEC)"
tests[36] = "$(EXIF.DATE.REGIONAL)"  -- Not Implemented
tests[37] = "$(EXIF.TIME.REGIONAL)"  -- Not Implemented
tests[38] = "$(EXIF.ISO)"
tests[39] = "$(EXIF.EXPOSURE)"
tests[40] = "$(EXIF.EXPOSURE.BIAS)"  -- Not Implemented
tests[41] = "$(EXIF.APERTURE)"
tests[42] = "$(EXIF.FOCAL.LENGTH)"
tests[43] = "$(EXIF.FOCUS.DISTANCE)"
tests[44] = "$(LONGITUDE)"
tests[45] = "$(LATITUDE)"
tests[46] = "$(ALTITUDE)"
tests[47] = "$(STARS)"
tests[48] = "$(RATING.ICONS)"        -- Not Implemented
tests[49] = "$(LABELS)"
tests[50] = "$(LABELS.ICONS)"       -- Not Implemented 
tests[51] = "$(MAKER)"
tests[52] = "$(MODEL)"
tests[53] = "$(TITLE)"
tests[54] = "$(DESCRIPTION)"
tests[55] = "$(CREATOR)"
tests[56] = "$(PUBLISHER)"
tests[57] = "$(RIGHTS)"
tests[58] = "$(TAGS)"                -- Not Implemented
tests[59] = "$(CATEGORY)"            -- Not Implemented
tests[60] = "$(SIDECAR_TXT)"         -- Not Implemented
tests[61] = "$(FOLDER.PICTURES)"
tests[62] = "$(FOLDER.HOME)"
tests[63] = "$(FOLDER.DESKTOP)"
tests[64] = "$(OPENCL.ACTIVATED)"    -- Not Implemented
tests[65] = "$(USERNAME)"
tests[66] = "$(NL)"
tests[67] = "$(JOBCODE)"    
tests[68] = "$(USERNAME^)" 
tests[69] = "$(USERNAME^^)" 
tests[70] = "$(MAKER,)"
tests[71] = "$(MAKER,,)"
tests[72] = "$(MAKER:0:5)"
tests[73] = "$(MAKER:-1:-11)"
tests[74] = "$(MAKER:6)"
tests[75] = "$(MAKER:-6)"
tests[76] = "$(USERNAME//l/f)" 
tests[77] = "$(ALTITUDE-$(FOLDER.HOME))"
tests[78] = "$(ALTITUDE-just a string)"
tests[79] = "$(USERNAME+johnbob)"
tests[80] = "$(FOLDER.HOME#/home/)"
tests[81] = "$(FOLDER.HOME%bill)"
tests[82] = "$(USERNAME/l/f)" 
tests[83] = "$(MAKER/#NIKON/CANON)"
tests[84] = "$(MAKER/%CORPORATION/HOBBY SHOP)"

expected[1] = darktable_picture_folder --"$(ROLL.NAME)"
expected[2] = darktable_picture_folder --"$(FILE.FOLDER)"
expected[3] = "0.Default.Image.NEF" -- "$(FILE.NAME
expected[4] = "NEF" --"$ (FILE.EXTENSION)"
expected[5] = "1" --"$(ID)"
expected[6] = "0" --"$(VERSION)"
expected[7] = "" --"$(VERSION.IF.MULTI)"    -- Not Implemented
expected[8] = "" --"$V(ERSION.NAME)"
expected[9] = "4.3.0+1373~g9301616ec0-dirty" -- "$(DARKTABLE.VERSION)"
expected[10] = ""      -- "$(DARKTABLE.NAME)" -- Not Implemented
expected[11] = "1" --"$(SEQUENCE)"
expected[12] = "7424"  --"$(WIDTH.SENSOR)"      -- Not Implemented
expected[13] = "4924"  --"$(HEIGHT.SENSOR )"    -- Not Implemented
expected[14] = "0"  --"$(WIDTH.RAW )"        -- Not Implemented
expected[15] = "0"  --"($HEIGHT.RAW)"        -- Not Implemented
expected[16] = "7374"  --"$(WIDTH.CROP)"        -- Not Implemented
expected[17] = "4924"  --"$(HEIGHT.CROP )"      -- Not Implemented
expected[18] = "7374"  --"$(WIDTH.EXPORT)"      -- Not Implemented
expected[19] = "4924"  --"$(HEIGHT.EXPORT)"     -- Not Implemented
expected[20] = ""  --"$(WIDTH.MAX)"         -- Not Implemented
expected[21] = ""  --"$(HEIGHT.MAX)"       -- Not Implemented
expected[22] = string.format("%4d", datetime.year) --"$(YEAR)"
expected[23] = string.format("%02d", datetime.month) --"$(MONTH)"
expected[24] = string.format("%02d", datetime.day) --"$(DAY)"
expected[25] = string.format("%02d", datetime.hour) --"$HOUR)"
expected[26] = string.format("%02d", datetime.min) --"$M(INUTE)"
expected[27] = string.format("%02d", datetime.sec) --"$(SECOND)"
expected[28] = "" --"$(MSEC)"
expected[29] = eyear --"$(EXIF.YEAR)"
expected[30] = emon --"$(EXIF.MONTH)"
expected[31] = eday --"$(EXIF.DAY)"
expected[32] = ehour --"$(EXIF.HOUR)"
expected[33] = emin --"$(EXIF.MINUTE)"
expected[34] = esec --"$(EXIF.SECOND)"
expected[35] = emsec --"$(EXIF.MSEC)"
expected[36] = "" --"$(EXIF.DATE.REGIONAL)"  -- Not Implemented
expected[37] = "" --"$(EXIF.TIME.REGIONAL)"  -- Not Implemented
expected[38] = "100" --"$(EXIF.ISO)"
expected[39] = "1/250" --"$(EXIF.EXPOSURE)"
expected[40] = "-1.0" --"$(EXIF.EXPOSURE.BIAS)"
expected[41] = "5.6" --"$(EXIF.APERTURE)"
expected[42] = "105" --"$(EXIF.FOCAL.LENGTH)"
expected[43] = "0.0" --"$(EXIF.FOCUS.DISTANCE)"
expected[44] = "" --"$(LONGITUDE)"
expected[45] = "" --"$(LATITUDE)"
expected[46] = "" --"$(ALTITUDE)"
expected[47] = "1" --"$(STARS)"
expected[48] = "" --"$(RATING.ICONS)"        -- Not Implemented
expected[49] = "" --"$(LABELS"
expected[50] = "" --"$(LABELS.ICONS)"       -- Not Implemented 
expected[51] = "NIKON CORPORATION" --"$(MAKER)"
expected[52] = "NIKON D800E" --"$(MODEL)"
expected[53] = "" --"$(TITLE)"
expected[54] = "" --"$(DESCRIPTION)"
expected[55] = "" --"$(CREATOR)"
expected[56] = "" --"$(PUBLISHER)"
expected[57] = "" --"$(RIGHTS)"
expected[58] = "" --"$(TAGS)"                -- Not Implemented
expected[59] = "" --"$(CATEGORY)"            -- Not Implemented
expected[60] = "" --"$(SIDECAR_TXT)"         -- Not Implemented
expected[61] = "/home/bill/Pictures" --"$(FOLDER.PICTURES)"
expected[62] = "/home/bill" --"$(FOLDER.HOME)"
expected[63] = "/home/bill/Desktop" --"$(FOLDER.DESKTOP)"
expected[64] = "" --"$(OPENCL.ACTIVATED)"    -- Not Implemented
expected[65] = "bill" --"$(USERNAME)"
expected[66] = "\n" --"$(NL)"
expected[67] = "" --"$(JOBCODE)"  
expected[68] = "Bill"     
expected[69] = "BILL"
expected[70] = "nIKON CORPORATION"
expected[71] = "nikon corporation"
expected[72] = "NIKON"
expected[73] = "CORPORATION"
expected[74] = "CORPORATION"
expected[75] = "RATION"
expected[76] = "biff"
expected[77] = "/home/bill"
expected[78] = "just a string"
expected[79] = "johnbob"
expected[80] = "bill"
expected[81] = "/home/"
expected[82] = "bifl"
expected[83] = "CANON CORPORATION"
expected[84] = "NIKON HOBBY SHOP"



local passed_tests = 0
-- change to log.info to see LOTS of output
ds.log_level = log.error
local passfail = "FAIL"
local home = "/home/bill"
local username = "bill"
local pic_folder = "/home/bill/Pictures"
local desktop = "/home/bill/Desktop"
for i = 1, #tests do
  dt.print_log("\nRunning test " .. i)
  local str = ds.substitute(img, 1, tests[i], username, pic_folder, home, desktop)
  -- local str = ds.substitute(img, 1, tests[i])
  if str == expected[i] then
    passfail = "pass"
    passed_tests = passed_tests + 1
  else
    passfail = "FAIL"
  end
  dt.print_log("Test " .. i .. " Result: " .. passfail .. " test: " .. tests[i] .. " Result: " .. str .. " Expected: " .. expected[i])
end

dt.print_log(passed_tests .. " tests passed")
